#include "config.h"

/* SecuMod version */
char *version="v2.0";

/* import for syscall table manipulation */
extern void *sys_call_table[];

/*
  which security features are on, these variables are especially for later
  extensions of SecuMod
*/
int trust_exec_protection_on = 0;
int trust_suid_protection_on = 0; /* */
int proc_fs_protection_on = 0;
int hardlink_protection_on = 0;
int symlink_protection_on = 0;
int rawdisk_protection_on = 0; /* breaks some fs-autosensing, mtools */
int pipe_protection_on = 0;
int trace_protection_on = 0;
int init_module_protection_on = 0;
int socket_protection_on = 0;
int cap_setuid_protection_on=0; /* */

/* command line options for security features */
int texec = 0;
int procfs = 0;
int hardlink = 0;
int symlink = 0;
int rawdisk = 0;
int pipe = 0;
int trace = 0;
int systable = 0;
int socket = 0;
int suid = 0;
int unremoveable = 0;
int capset = 0; /* */

/* command line option for logging */
int logging = 0;

/* command line option for cap_bset value */
char *capbits;

/* for restoring the old cap_bset value (the value before insmod'ing */
kernel_cap_t cap_old; 

/* for path table */
static int path_count;
static int suid_count;

/* this is for checking system call manipulation */
void *sys_call_table_backup[256];
volatile char sys_call_hooked[256] = { 0, };

static char * texec_paths[NR_TPATHS] = { NULL, };
static char * suid_paths[NR_TPATHS] = { NULL, };


mm_segment_t old_fs;
void begin_kmem() { old_fs = get_fs(); current->addr_limit = KERNEL_DS; };
void end_kmem()   { current->addr_limit = old_fs; };


void secu_alert(const char * reason, const char * detail)
  /* TODO: sanity checking for the pointers and strings if in kernel space */
{
	int error;
	char use_detail_k;
	char use_reason_k;
	static char * noaccess = "<READ ERROR>";
	static char * nostring = "<NIL>";
	char * detail_k;
	char * reason_k;
  
  	use_detail_k = 0;
  	use_reason_k = 0;
  	
	if (!reason) 
	{ 
	  	reason_k = nostring; 
	} else
	{ 
	  	if (reason <  (char *)0xc0001000L)
	  	{
	  		reason_k = getname(reason);
		  	error = PTR_ERR(reason_k);
		  	if (IS_ERR(reason_k))
		  	{	
		  		reason_k = noaccess;
		  	} else
		  	{
		  		use_reason_k = 1;
		  	};
		}
		else reason_k = reason;
	};

	if (!detail) 
	{ 
	  	detail_k = nostring; 
	} else
	{ 
	  	if (detail <  (char *)0xc0001000L)
	  	{
	  		detail_k = getname(detail);
		  	error = PTR_ERR(detail_k);
		  	if (IS_ERR(detail_k)) 
		  	{	
		  		detail_k = noaccess;
		  	} else
		  	{
		  		use_detail_k = 1;
		  	};
		}
		else detail_k = detail;
	};

    
	printk(REAL_LOG "SecuMod: %.15s PID %u UID %u %.80s (%.80s)\n",
	       current->comm, current->pid, current->uid, reason_k, detail_k);
         
	if ((current->euid!=current->uid) || (current->fsuid!=current->uid) ||
	    (current->suid!=current->uid) ||
	    (current->egid!=current->gid) || (current->fsgid!=current->gid) ||
	    (current->sgid!=current->gid))
		printk(REAL_LOG "SecuMod: (%u) UID/EUID/FSUID/SUID are "
		       "%d/%d/%d/%d, *GID are %d/%d/%d/%d "
                       "CAP_{EFF,INHER,PERM} are %x,%x,%x\n", current->pid,
		current->uid, current->euid, current->fsuid, current->suid,
		current->gid, current->egid, current->fsgid, current->sgid,
		current->cap_effective, current->cap_inheritable,
		current->cap_permitted);

 	if (use_detail_k) putname(detail_k);
 	if (use_reason_k) putname(reason_k);
  
	return;
};
