int (*orig_execve) (struct pt_regs regs);

asmlinkage int hacked_execve(struct pt_regs regs) /* */
{
	char filename_k[PATH_MAX]; /* */
	char * real_path; /* */
	char *filename;
	struct dentry * dentry;
	unsigned int counter;
	int error = 0;
	int is_suid = 0;

        lock_kernel(); /* */
#ifndef SPEED_UP_EXEC
        if (systable) check_sys_call_table_backup(0); // 0 = not verbose
#endif        
        
	filename = getname((char*)regs.ebx);
	filename_k[0] = '\0';
	error = PTR_ERR(filename);
 
	if (IS_ERR(error)) goto leave;

        dentry = open_namei(filename, 0, 0);
          /* uses lookup_dentry(pathname, base (is NULL here), ...) */
          /* lookup_dentry uses base, and if none is given, it uses */
          /* current->fs->root (*pathname is /) or current->fs->pwd  */
        error = PTR_ERR(dentry);
        if (IS_ERR(dentry)) goto leave2; // file not found
        
	/* get path and S_ISUID of dentry -- /usr/include/linux/dcache.h */
	real_path = d_path(dentry,filename_k,sizeof(filename_k)-1);
	is_suid = dentry->d_inode->i_mode & (S_ISUID | S_ISGID);

	printk(NOTIFY_LOG "execve: %.15s of %d does (is_suid=%d) %.240s\n",
	       current->comm, current->uid, is_suid, real_path);

	if (current->uid != 0)
	{ if (trust_exec_protection_on) 
	  {	for (counter=0; fixed_tpaths[counter]; counter++)
	  	if (fixed_tpaths[counter])
		{ if (strncmp(real_path, fixed_tpaths[counter], 
			      strlen(fixed_tpaths[counter])) == 0)
		    goto ok1; 
		};
		for (counter=0; counter < NR_TPATHS; counter++) 
		if (texec_paths[counter]) 
		{ if (strncmp(real_path, (texec_paths[counter]), 
			      strlen(texec_paths[counter])) == 0) 
		    goto ok1; 
		};
		if (logging) secu_alert("tried to execute",real_path);
#ifndef EXEC_LOGONLY
		dput(dentry);	// NEW
        	putname(filename);
		unlock_kernel();
		return -1;
#endif
	  	ok1:
	  }; // end_exec_check

	  if (trust_suid_protection_on && is_suid)
	  {	for (counter=0; fixed_suidpaths[counter]; counter++)
	  	if (fixed_suidpaths[counter])
		{ if (strncmp(real_path, fixed_suidpaths[counter], 
			      strlen(fixed_suidpaths[counter])) == 0)
		    goto ok2; 
		};
		for (counter=0; counter < NR_TPATHS; counter++) 
		if (suid_paths[counter]) 
		{ if (strncmp(real_path, (suid_paths[counter]), 
			      strlen(suid_paths[counter])) == 0) 
		    goto ok2; 
		};
		if (logging) secu_alert("tried to execute SUID/SGID",real_path);
#ifndef EXEC_LOGONLY
		dput(dentry);	// NEW
        	putname(filename);
		unlock_kernel();
		return -1;
#endif
		ok2:
	  }; // end_suid_check
	}; // end_is_root

	error = do_execve(filename, (char**)regs.ecx, (char**)regs.edx, &regs);
        if (error == 0) current->flags &= ~PF_DTRACE; /* as in process.c */
        dput(dentry);
        leave2:
        putname(filename);
	leave:
	unlock_kernel();
	return error;
};


int execve_hooks = 0;


void register_execve(void)
{
	printk(NOTIFY_LOG "SecuMod: adding hook to execve (%d hooks before)\n",execve_hooks);
	if (execve_hooks==0)
	{
		orig_execve = sys_call_table[SYS_execve];
		sys_call_table[SYS_execve] = hacked_execve;
		sys_call_hooked[SYS_execve] = 1;
	};
	execve_hooks++;
	return;
};


void deregister_execve(void)
{
	printk(NOTIFY_LOG "SecuMod: removing 1 of %d hooks to execve\n",execve_hooks);
	execve_hooks--;
	if (execve_hooks==0)
	{
		sys_call_table[SYS_execve] = orig_execve; 
		sys_call_hooked[SYS_execve] = 0;
	};
	return;
};


void exec_trust_protection(int on)
  /* TODO: make this work ok for SMP */
{
	int n;
	
	for (n=0; n<NR_TPATHS; n++) texec_paths[n] = NULL;
	/* if switch goes on -> off, memory should have been freed here! */
	
	trust_exec_protection_on = on;
	if (on) { register_execve(); } else { deregister_execve(); };
	return;
};

void suid_trust_protection(int on)
  /* TODO: as above */
{
	int n;
	
	for (n=0; n<NR_TPATHS; n++) suid_paths[n] = NULL;
	/* if switch goes on -> off, memory should have been freed here! */
	
	trust_suid_protection_on = on;
	if (on) { register_execve(); } else { deregister_execve(); };
	return;
};
