volatile int open_hooks = 0;

int (*orig_open)(const char *pathname, int flag, mode_t mode);

int hacked_open(const char *pathname, int flag, mode_t mode)
{
	char * pathname_k;
	struct dentry *dent;
	int error;
	int is_proc;

	pathname_k = getname(pathname);
	error = PTR_ERR(pathname_k);
	if (IS_ERR(pathname_k)) 
	{
		printk(NOTIFY_LOG "SecuMod: open: cannot getname pathname.\n");
		goto leave_open;
	};
	                 
	is_proc = (strncmp("/proc", pathname_k, strlen("/proc")) == 0);
	if (is_proc) 
	{
// --		printk(NOTIFY_LOG "SecuMod: open: no check, is in /proc: %.240s.\n",pathname_k);
		goto ok_open; // did not even try to find the dentry :(
	};
 
	dent = open_namei(pathname_k, 0, 0); 		// lnamei(pathname_k) ?
	if (IS_ERR(dent))
	{ 
// --		printk(NOTIFY_LOG "SecuMod: open: cannot find dentry for %.240s.\n",pathname_k);
		goto ok_open;	
	};
 	
	/* TODO: / QUESTION: (because this breaks some fs-stuff) */
	/* how about an exception for group disk, root or the kernel? */
	if (rawdisk_protection_on)
	if (S_ISBLK(dent->d_inode->i_mode)) 
/*      if (current->uid != 0)		*/
	/* deny any block device access */
	{
		if (logging) secu_alert("tried to open block device",
					pathname_k);
		error = -1;
		dput(dent);	// NEW
		goto err_open;
	};
	
// --	printk(NOTIFY_LOG "SecuMod: open: passed rawdisk_protection_on=%d: %.240s.\n", rawdisk_protection_on,pathname_k);

        if (pipe_protection_on)
	if (S_ISFIFO(dent->d_inode->i_mode))
        if (current->uid != 0)
        if (dent->d_inode->i_uid != current->fsuid)
        if (dent->d_inode->i_uid != 0)
        {
        if (dent->d_inode->i_uid != POSTFIX)
	{
                if (logging) secu_alert("tried to open pipe",pathname_k);
		error = -1;
		dput(dent);	// NEW
		goto err_open;
	} else 
	printk(NOTIFY_LOG "SecuMod: special permission to open pipe owned by Postfix\n");
	};
	
// --	printk(NOTIFY_LOG "SecuMod: open: passed pipe_protection_on=%d: %.240s.\n", pipe_protection_on,pathname_k);

	/* TODO: open_namei is the best for pipe and raw protection, 		*/
	/*       BUT as it follows links, it knocks out any symlink alert! 	*/

	dput(dent);	// NEW
	dent = lnamei(pathname); // NOT pathname_k here
	if (IS_ERR(dent))
	{ 
// -- 		( enable this printk to see more file not found errors: )
		printk(NOTIFY_LOG "SecuMod: open: cannot lnamei dentry for %.240s.\n",pathname_k);
		goto ok_open;	
	};

	if (symlink_protection_on)
        if (S_ISLNK(dent->d_inode->i_mode)) 
/*      if (current->uid != 0)	*/
        if (dent->d_inode->i_uid != current->fsuid)  
        if (dent->d_inode->i_uid != 0) 
	{
                if (logging) secu_alert("tried to open symlink", pathname_k);
		error = -1;
		dput(dent);	// NEW
		goto err_open;
	};

// --	printk(NOTIFY_LOG "SecuMod: open: allowed for %.240s.\n", pathname_k);

	dput(dent);	// NEW
	ok_open:
	error = orig_open(pathname, flag, mode); // NOT pathname_k but pathname!
		// if you use pathname_k, EFAULT is returned (see asm/errno.h)
// --	printk(NOTIFY_LOG "SecuMod: open: returned %d for %.240s.\n", error, pathname_k);

	err_open:
	putname(pathname_k); // putname serves to release kernel memory
	leave_open:
	return error;
};


void register_open(void)
{
	printk(NOTIFY_LOG "SecuMod: adding hook to open (%d hooks before).\n", open_hooks);
	if (open_hooks==0)
	{
		orig_open = sys_call_table[SYS_open];
		sys_call_table[SYS_open] = hacked_open;
		sys_call_hooked[SYS_open] = 1;
	};
	open_hooks++;
	return;
};


void deregister_open(void)
{
	printk(NOTIFY_LOG "SecuMod: removing 1 of %d hooks to open.\n", open_hooks);
	open_hooks--;
	if (open_hooks==0)
	{
		sys_call_table[SYS_open] = orig_open; 
		sys_call_hooked[SYS_open] = 0;
	};
	return;
};


void rawdisk_protection(int on)
{
	rawdisk_protection_on = on;
	if (on) { register_open(); } else { deregister_open(); };
	return;
};


void pipe_protection(int on)
{
        pipe_protection_on = on;
	if (on) { register_open(); } else { deregister_open(); };
	return;
};



void symlink_protection(int on)
{
	symlink_protection_on = on;
	if (on) { register_open(); } else { deregister_open(); };
	return;
};
