/*
 * SecuMod	Is a Security Module for the LINUX Kernel >= 2.2.14, which
 *		increase the system security via mapping system calls.
 *
 *
 * Version:	@(#)secumod.c	0.9	05/21/2000
 *
 * Authors:	Mark Vogelsberger, <Mark.Vogelsberger@rz.uni-karlsruhe.de>
 *		Thomas Biege, <thomas@suse.de>
 *		Eric Auer, <eric@CoLi.Uni-SB.de>
 *		Tobias Koenig, <tokoe82@yahoo.de>
 *
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * History:
 *		XX jan 199?
 *			wrote the basic code
 *
 *		27 apr 2000
 *			add proc device for changing trusted paths
 *		
 *		16 may 2000
 *			integrate socketcall filter
 *
 *		19 jun 2000
 *			bugfixes in device.h and execprot.h
 *
 *              2* jul 2000
 *                      bugfixes all around the place, files reorganized
 */

#include <linux/module.h>
#include <linux/malloc.h>
#include <linux/capability.h>
#include <asm/uaccess.h>
#include <sys/syscall.h>

#include <linux/smp_lock.h> 
  /* [un]lock_kernel - not yet inserted at all places! */

#include "config.h"
#include "common.h"

/* include files for security features */
#include "procprot.h"	/* applies chmod on /proc */
#include "sysprot.h"	/* ex-modprot now included in sysprot */
#include "execprot.h"	/* fixed for real path - also activates sysprot now */
			/* suidprot now included in execprot */
#include "hardprot.h"	/* hardlink protection */
#include "openprot.h"	/* symprot, rawprot, pipeprot (for sys_open) */
#include "traceprot.h"	/* disables all kinds of sys_trace calls */
#include "sockprot.h"	/* logs socket events */
#include "cap_prot.h"	/* CAP_SETUID fix (needed for Linux 2.2.[0-15]) */

#ifdef PROC_INTERFACE
#include "device.h"	/* a /proc interface for execprot paths */
#endif

/* module load parameter for every security feature */
MODULE_PARM(texec, "i");
MODULE_PARM(procfs, "i");
MODULE_PARM(hardlink, "i");
MODULE_PARM(symlink, "i");
MODULE_PARM(rawdisk, "i");
MODULE_PARM(pipe, "i");
MODULE_PARM(trace, "i");
MODULE_PARM(systable, "i");
MODULE_PARM(socket, "i");
MODULE_PARM(suid, "i");
MODULE_PARM(unremoveable, "i");
MODULE_PARM(capset, "i"); /* */
MODULE_PARM(logging, "i");

/* which capbits setting */
MODULE_PARM(capbits, "s");

extern kernel_cap_t cap_bset; 

/* MODULE INIT */
int init_module(void)               
{
	int counter = 0;

#ifndef DO_EXPORT_SYMBOLS
	EXPORT_NO_SYMBOLS;
#endif

	if (unremoveable) MOD_INC_USE_COUNT;

	printk(REAL_LOG "SecuMod: %s start-up Mark.Vogelsberger@rz.uni-karlsruhe.de\n",version);
        printk(REAL_LOG "SecuMod: Options "
                            "trace=%d texec=%d suid=%d systable=%d pipe=%d "
                            "rawdisk=%d hardlink=%d symlink=%d procfs=%d "
                            "socket=%d capset=%d unremoveable=%d logging=%d\n",
                            trace, texec, suid, systable, pipe, rawdisk,
                            hardlink, symlink, socket,
                            procfs, capset, unremoveable, logging);

	if (trace) {
		printk(START_LOG "SecuMod: Process Trace Protection  : ON\n");
		trace_protection(1);  
	}; 

	if (texec) {
		path_count = 0;
		printk(START_LOG "SecuMod: Trusted Path Protection   : ON\n");
		exec_trust_protection(1); 
	};

	if (systable) {
		printk(START_LOG "SecuMod: SysCall Table Checking    : ON\n");
		init_module_protection(1);
	};

	if (pipe) {
		printk(START_LOG "SecuMod: Named Pipe Protection     : ON\n");
		pipe_protection(1);
	};

	if (rawdisk) {
		printk(START_LOG "SecuMod: RawDisk Access Protection : ON\n");
		rawdisk_protection(1);
	};

	if (hardlink) {
		printk(START_LOG "SecuMod: HardLink Create Protection: ON\n");
		hardlink_protection(1);
	};

	if (symlink) {
		printk(START_LOG "SecuMod: SymLink Follow Protection : ON\n");
		symlink_protection(1);
	};

	if (procfs) {
		printk(START_LOG "SecuMod: ProcFS rights changed     : ON\n");
		proc_fs_protection(1);
	};

	if (socket) {
		printk(START_LOG "SecuMod: Socket Logging            : ON\n");
		socket_protection(1);
	};

	if (suid) {	
		suid_count = 0;
		printk(START_LOG "SecuMod: Trusted Suid Protection   : ON\n");
		suid_trust_protection(1);
	};


#ifdef SYSPROT_TEST_MODE
	sys_call_table[234] = make_sys_call_table_backup; // uebler Hack, aber funktioniert!
	// nur eine helper-app die sc 234 ruft crasht, nicht mehr insmod :-)
#else
	if (systable) make_sys_call_table_backup();
#endif

	cap_old = cap_bset;
	if (capbits != NULL) {
		while(cap_bset_modes[counter].desc != NULL) {
			if (strncmp(capbits, cap_bset_modes[counter].desc,
			strlen(cap_bset_modes[counter].desc)) == 0) {
				cap_bset = cap_bset_modes[counter].cap_val;
				printk(REAL_LOG "SecuMod: CapBits Setting"
				       "           : %s[%u, 0x%0x]\n", 
				       capbits, cap_bset,cap_bset);
			printk(NOTIFY_LOG "SecuMod: Old CapBits were 0x%x\n",
			       cap_old);
			};
			counter++;
		};
	};

	
#ifdef PROC_INTERFACE
	return proc_register(&proc_root, &secumod_procfile);
#else
	return 0;
#endif

};

/* MODULE CLEANUP - not reached if the module is unremoveable */
void cleanup_module(void)           
{
	int n;

	printk(NOTIFY_LOG "SecuMod: releasing all hooked syscalls\n");
	cap_bset = cap_old;

	if (systable) init_module_protection(0);
	if (procfs) proc_fs_protection(0);
	if (hardlink) hardlink_protection(0);
	if (symlink) symlink_protection(0);
	if (rawdisk) rawdisk_protection(0);
	if (pipe) pipe_protection(0);
	if (texec) exec_trust_protection(0);
	if (trace) trace_protection(0);
	if (socket) socket_protection(0);
	if (suid) suid_trust_protection(0);
        if (capset) cap_setuid_protection(0);

	for (n=0; n<256; n++) if (sys_call_hooked[n])
		printk(REAL_LOG "SecuMod: prepare for chaos, "
		                "clean-up did not release syscall %d\n",
		       n);
	
	printk(REAL_LOG "SecuMod: %s clean-up\n",version);

#ifdef PROC_INTERFACE
	proc_unregister(&proc_root, secumod_procfile.low_ino);
#endif

	return;
};
