/*
 * SecuTool	Is a tool for changing secumod's trusted paths over a
 *		/proc device.
 *
 * Version:	@(#)secutool.c	2.0	05/21/2000
 *
 * Author:	Tobias Koenig, <tokoe82@yahoo.de>
 *
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 */

#include <stdio.h>
#include <fcntl.h>

#include "request.h"

void usage()
{
	fprintf(stderr, "usage: secutool -[TS] -[adcs] <path>\n"
			"T\t operations on texec path\n"
			"S\t operations on suid path (not implemented)\n\n"
			"a\t add the given path\n"
			"d\t remove the given path\n"
			"c\t clear the whole table (but not fixed paths)\n"
			"s\t print all table entries via the syslog facility\n\n");
}

int main(int argc, char *argv[])
{
	struct path_entry entry;
	int fd;
    
	if (argc < 3) {
		usage();
		return -1;
	}

	if ((fd = open("/proc/secumod", O_WRONLY)) < 0) {
		fprintf(stderr, "can't open /proc/secumod\n");
		return -1;
	}

	bzero(entry.path, '\0', 256);

	if (strcmp(argv[1], "-T") == 0)
		entry.request_type = TEXEC_REQUEST;

	if (strcmp(argv[1], "-S") == 0)
		entry.request_type = SUID_REQUEST;
		
	if (strcmp(argv[2], "-a") == 0) {
		if (argc != 4) {
			usage();
			return -1;
		}
		entry.type = PATH_INSERT;
		memcpy(entry.path, argv[3], strlen(argv[3]));
		entry.path[strlen(argv[3])] = '\0';
	}

	if (strcmp(argv[2], "-d") == 0) {
		if (argc != 4) {
			usage();
			return -1;
		}
		entry.type = PATH_REMOVE;
		memcpy(entry.path, argv[3], strlen(argv[3]));
		entry.path[strlen(argv[3])] = '\0';
	}

	if (strcmp(argv[2], "-c") == 0) {
		entry.type = PATH_CLEAR;
	}

	if (strcmp(argv[2], "-s") == 0) {
		entry.type = PATH_SHOW;
	}

    
	write(fd, &entry, sizeof(struct path_entry));
	close(fd);

	return 0;
}